;;; *******************************************************************
;;; * Autor         : Dipl. Ing. Jan Tappenbeck                       *
;;; * Projektname   : MultiOffset                                     *
;;; * fr AutoCAD   : R14/R2000/R2002                                 *
;;; *******************************************************************
;;; * Versionsnummer: 1.0                                             *
;;; * Datei         : MULTIOFFSET.LSP                                 *
;;; * zugehrige DCL: - entfllt -                                    *
;;; * zugehrige SLD: - entfllt -                                    *
;;; * Sprache       : deutsch / international                         *
;;; * nderungen    : ./.                                             *
;;; *******************************************************************
;;; * Kurzbeschreibung:                                               *
;;; * mehrfaches Versetzen von Objekten                               *
;;; *******************************************************************
;;; * Globale Variablen                                               *
;;; * *MV_COUNT*, *MV_DIST*                                           *
;;; *******************************************************************
;;; * (c) Jan Tappenbeck, 10/2001, Germany                            *
;;; * jan.tappenbeck@t-online.de                                      *
;;; *******************************************************************
;;; *                                                                 *
;;; * Lizenz:                                                         *
;;; * Das Programm wird als Freeware-Version freigegeben !            *
;;; *                                                                 *
;;; * Fr Schden, die durch die Anwendung dieses Programmes          *
;;; * entstehen, kann keine Haftung bernommen werden !               *
;;; *                                                                 *
;;; *******************************************************************
;;; *                                                                 *
;;; * Disclaimer:                                                     *
;;; * This software is provided "as is" without express or            *
;;; * implied warranty.                                               *
;;; *                                                                 *
;;; *******************************************************************

(DEFUN C:mv (/		old_err	   tmp_dist   tmp_count	 idx
	     side_point	entset	   entget_flag		 ELIST
	     LAYER	LLIST	   FLAGS      TYP	 dummy
	     Element	elem_dist
	     )


  (modes				;speichern der Systemvariablen
    '("CMDECHO"
      "OSMODE"
      "BLIPMODE"
      "DIMZIN"
      )
    )
  (SETVAR "CMDECHO" 0)
  (COMMAND "_.UNDO" "_Begin")
  (SETVAR "OSMODE" 0)
  (SETVAR "BLIPMODE" 0)
  (SETVAR "DIMZIN" 3)			;Nachkomma-Nullen anzeigen

  (SETQ
    old_err *error*			;setzen der internen Fehlerroutine
    *error* mv_err
    )					;end-setq


  (princ *MV_MSG2*)
  ;;Versetzen - mehrfach


  ;; Abstand abfragen

  (initget (+ 2 4))			;keine Null und negative Zahlen
  (if (not *MV_DIST*)			;wenn noch kein Wert bekannt
    (setq *MV_DIST* 1.0)
    )					;endif
  (if
    (setq tmp_dist
	   (getdist
	     (strcat "\n\n"
		     *MV_MSG3*
		     " <"		;Abstand angeben
		     (rtos *MV_DIST* (getvar "LUNITS") (getvar "LUPREC"))
		     ">: "
		     )			;end-strcat
	     )				;end-getdist
	  )				;end-setq
     (setq *MV_DIST* tmp_dist)		;wenn neuer Wert eingegeben
     )					;endif
  ;; Anzahl abfragen

  (initget (+ 2 4))			;keine Null und negative Zahlen
  (if (not *MV_COUNT*)			;wenn noch kein Wert bekannt
    (setq *MV_COUNT* 1)
    )					;endif
  (if
    (setq tmp_count
	   (getint (strcat *MV_MSG4*
			   " <"		;Anzahl der Versetzungen angeben
			   (itoa *MV_COUNT*)
			   ">: "
			   )		;end-strcat
		   )			;end-getdist
	  )				;end-setq
     (setq *MV_COUNT* tmp_count)	;wenn neuer Wert eingegeben
     )					;endif

  ;; Objekt abfragen

  (setq entget_flag nil)
  (while (not entget_flag)
    (princ *MV_MSG5*)			;Zu versetzendes Objekt whlen: 
    (setq entset (entsel))		;Element abfragen

    (if	entset				;wenn Element gewhlt
      (progn
	;;Element gesperrt ?
	(SETQ ELEMENT (car entset)
	      ELIST   (ENTGET ELEMENT)
	      LAYER   (CDR (ASSOC 8 ELIST))
	      TYP     (CDR (ASSOC 0 ELIST))
	      LLIST   (TBLSEARCH "LAYER" LAYER)
	      FLAGS   (CDR (ASSOC 70 LLIST))
	      )

	(IF (> (LOGAND FLAGS 4) 0)	;Element gesperrt
	  (princ *MV_MSG6*)		;Das Objekt ist auf einem gesperrten Layer.
	  (if
	    (or
	      (= TYP "CIRCLE")
	      (= TYP "LINE")
	      (= TYP "POLYLINE")
	      (= TYP "LWPOLYLINE")
	      (= TYP "ARC")
	      (= TYP "XLINE")
	      (= TYP "SPLINE")
	      )				;end-OR
	     (setq entget_flag T)	;Typ zulssig
	     (princ *MV_MSG7*)		;Dieses Objekt kann nicht versetzt werden.
	     )				;endif
	  )				;endif Element gesperrt

	)				;end-progn
      )					;endif

    )					;end-while

  ;; Seite abfragen

  (setq	side_point
		   (getpoint
		     *MV_MSG8*		;Punkt auf Seite angeben, auf die versetzt werden soll:
		     )
	elem_dist  *MV_DIST*
	)

  ;; Zeichnen der neuen Objekte

  (repeat *MV_COUNT*
    (command "_.offset" elem_dist ELEMENT side_point "")
    (setq dummy	    (entlast)
	  elem_dist (+ elem_dist *MV_DIST*)
	  )				;end-setq
    )					;end-repeat

  (princ *MV_MSG9*)			;Das Element wurde versetz

  (SETQ *error* old_err)
  (COMMAND "_.UNDO" "_End")
  (moder)
  (PRINC)
  )




(DEFUN moder ()				; Rcksetzen der Systemvariablen - Unterprogramm
  (REPEAT (LENGTH mlst)
    (SETVAR (CAAR mlst) (CADAR mlst))
    (SETQ mlst (CDR mlst))
    )
  )

(DEFUN modes (a)			;speichern der Systemvariablen
  (SETQ mlst '())
  (REPEAT (LENGTH a)
    (SETQ mlst (APPEND mlst (LIST (LIST (CAR a) (GETVAR (CAR a))))))
    (SETQ a (CDR a))
    )
  )

(DEFUN mv_err (s)			;interne Fehlerroutine
  (IF (/= s *MV_MSG10*)			;Funktion abgebrochen
    (PRINC (STRCAT *MV_MSG11* s))	;Fehler
    )
  (SETQ *error* old_err)
  (COMMAND "_.UNDO" "_End")
  (COMMAND "_u")
  (moder)
  (PRINC)
  )

(cond
  ((= (getvar "locale") "DEU")
   (setq
     *MV_MSG1*
		(strcat	"\n\nVersetzen - mehrfach"
			"\n===================="
			"\nJan.Tappenbeck@t-online.de , 10/2001"
			"\nProgrammstart: MV"
			)		;end-strcat
     *MV_MSG2*	"\n\nVersetzen - mehrfach\n===================="
     *MV_MSG3*	"Abstand angeben"
     *MV_MSG4*	"Anzahl der Versetzungen angeben"
     *MV_MSG5*	"\nZu versetzendes Objekt whlen: "
     *MV_MSG6*	"\nDas Objekt ist auf einem gesperrten Layer."
     *MV_MSG7*	"\nDieses Objekt kann nicht versetzt werden."
     *MV_MSG8*	"\nPunkt auf Seite angeben, auf die versetzt werden soll: "
     *MV_MSG9*	"\nDas Element wurde versetz."
     *MV_MSG10*	"\nFunktion abgebrochen"
     *MV_MSG11*	"\nFehler: "
     )					;end-setq 
   )					;end-german


  (T					;ENGLISCH
   (setq
     *MV_MSG1*
		(strcat	"\n\noffset - multiple"
			"\n=================="
			"\njan.tappenbeck@t-online.de , 10/2001"
			"\ncommand: MV"
			)		;end-strcat
     *MV_MSG2*	"\n\noffset - multiple\n=================="
     *MV_MSG3*	"distance of offset"
     *MV_MSG4*	"count of offset"
     *MV_MSG5*	"\nselect object to offset: "
     *MV_MSG6*	"\nobject-layer is lock."
     *MV_MSG7*	"\nunpossible to offset object."
     *MV_MSG8*	"\npoint of side to  offset: "
     *MV_MSG9*	"\nobject was offset"
     *MV_MSG10*	"\nfunction cancel"
     *MV_MSG11*	"\nerror: "
    )					;end-setq
   )					;end-english


  )					;end-cond



(princ *MV_MSG1*)
(princ)
;|Visual LISP Format Options
(72 2 40 1 nil "end of " 60 9 0 0 0 T T nil T)
;*** KEINEN Text unterhalb des Kommentars hinzufgen! ***|;
